﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;

namespace Senior_Capstone
{
    public partial class MainForm : Form
    {
        private MyDocument myDoc;

        //Graph Creation variables:
        private Image img;
        private Graphics imgGraphics;
        private Pen myPen = new Pen(Color.Black, 2);
        private int edgeWidth = 5;
        const int MAXVERTICES = 30;
        const int VERTEXSIZE = 25;
        private Point first; //mouse down location
        private Point second; //mouse up location
        private Point mlocation; //current mouse location
        private bool directed = true;
        private int weight = 1;
        private bool mDown = false; //mouse down bool

        //Algorithm variables:
        private Color defaultColor = Color.Black;
        //Vertex colors
        private Color visitedColor = Color.Red;
        private Color doneColor = Color.Green;
        //Edge colors
        private Color treeEdgeColor = Color.RoyalBlue;
        private Color backEdgeColor = Color.DarkOrange;
        private Color forwardEdgeColor = Color.Chocolate;
        private Color crossEdgeColor = Color.DeepPink;
        private Color tempEdgeColor = Color.Gray;

        private bool runAlgorithm = false;
        private string algor;   //name of algorithm
        private int start = 0;  //vertex algorithm starts on
        private List<MyDocument.StepResult> stepList;
        private IEnumerator<MyDocument.StepResult> enumer;
        private MyDocument.StepResult curValue; //current values of algorithm
        private int step = 0; //current step number of algorithm


        private List<string> MRU;
        string mruFilePath;

        //Event Handlers
        public event MouseEventHandler AddPointEvent;
        public event MouseEventHandler AddEdgeEvent;
        public event MouseEventHandler MoveVertexEvent;
        public event MouseEventHandler DeleteVertexEvent;
        public event MouseEventHandler MouseDownEvent;
        public event EventHandler ClearGraphEvent;
        public event EventHandler ClearAlgorithmEvent;

        public MainForm()
        {
            InitializeComponent();
            myDoc = new MyDocument();
            myDoc.Host = this;

            img = new Bitmap(ClientRectangle.Width, ClientRectangle.Height);
            imgGraphics = Graphics.FromImage(img);

            this.myDoc.NewDocument += new System.EventHandler(this.viewNewDocument);
            this.myDoc.OpenDocument += new System.EventHandler(this.viewOpenDocument);
            this.myDoc.UpdateCaption += new System.EventHandler(this.viewUpdateCaption);

            AddPointEvent += new MouseEventHandler(myDoc.docMouseClick);
            AddEdgeEvent += new MouseEventHandler(myDoc.docAddEdge);
            MoveVertexEvent += new MouseEventHandler(myDoc.docVertexMove);
            DeleteVertexEvent += new MouseEventHandler(myDoc.docDeleteVertex);
            MouseDownEvent += new MouseEventHandler(myDoc.docMouseDown);
            ClearGraphEvent += new EventHandler(myDoc.docClearGraph);
            mruFilePath = Environment.CurrentDirectory + "\\MRU.txt";
            FillMRU();

            ClearAlgorithmEvent += new EventHandler(myDoc.docClearAlgorithm);
            

        }

        private void onPaint(object sender, PaintEventArgs e)
        {
            Graphics g = this.CreateGraphics();
            g.DrawImage(img, 0, 0);
            
        }

        //Draws a vertex given its location, color, and index number
        private void DrawVertex(int x, int y, Color c, int num)
        {
            myPen.Color = c;
            imgGraphics.DrawEllipse(myPen, x - (VERTEXSIZE / 2), y - (VERTEXSIZE / 2), VERTEXSIZE, VERTEXSIZE); //draw circle
            Font f = new Font("Ariel", 11);
            SolidBrush myBrush = new SolidBrush(c);
            imgGraphics.DrawString(num.ToString(), f, myBrush , x - (VERTEXSIZE / 3), y - (VERTEXSIZE / 3)); //draw number
            Invalidate();
        }

        //Draws an edge using the color of pen p and between points a and b
        private void DrawEdge(Pen p, Point a, Point b)
        {
            if (a != new Point(-1, -1) && b != new Point(-1, -1) && a != b) //checks that a and b are vertices and not the same vertex
            {
                Point c = a; //to not corrupts a's value - need it later (copy)
                Point d = b; //to not corrupts b's value - need it later (copy)
                double angle = Math.Atan2((b.Y - a.Y), (b.X - a.X)); //find angle of slope
                //points c and d are changed so they are close to the vertex but are not in the vertex
                c.X += (int)((2 * VERTEXSIZE / 3) * Math.Cos(angle));
                c.Y += (int)((2 * VERTEXSIZE / 3) * Math.Sin(angle));
                d.X -= (int)((2 * VERTEXSIZE / 3) * Math.Cos(angle));
                d.Y -= (int)((2 * VERTEXSIZE / 3) * Math.Sin(angle));
                imgGraphics.DrawLine(p, c, d); //draw edge starting at c and ending at d
                DrawEdgeWeight(p, a, b); //draw edge weight in middle of edge
            }
            Invalidate();
        }

        //Draw edge weight in center of respective edge
        private void DrawEdgeWeight(Pen p, Point a, Point b)
        {
            if (myPen.Color == defaultColor) //only draws when the edge is between two vertices
            {
                Font f = new Font("Ariel", 10, FontStyle.Bold);
                int h = (int)f.GetHeight(); //height in pixels of string (number) to be drawn
                //Find middle of edge - adjust to center text (h/2)
                int x = (a.X + b.X) / 2 - h / 2;
                int y = (a.Y + b.Y) / 2 - h / 2;
                int w = myDoc.AdjMatrixWeight(myDoc.VertexNumber(a.X, a.Y), myDoc.VertexNumber(b.X, b.Y)); //get weight of edge
                //String width in pixels - https://msdn.microsoft.com/en-us/library/9bt8ty58(v=vs.110).aspx
                SizeF stringSize = new SizeF();
                stringSize = imgGraphics.MeasureString(w.ToString(), f);
                SolidBrush myBrush = new SolidBrush(Color.White);
                imgGraphics.FillEllipse(myBrush, x, y, stringSize.Width, h); //Draw white oval background so text can be seen
                myBrush.Color = defaultColor;   //Set brush color to black
                imgGraphics.DrawString(w.ToString(), f, myBrush, x, y); //Draw edge weight number on edge
            }
        }

        //Clear and redraw whole graph - vertices, edges, then edge weights
        private void RedrawGraph()
        {
            imgGraphics.Clear(Color.Transparent); //Clear graph visually
            for (int i = 0; i < myDoc.VertexCount; i++) //draw vertices
            {
                myPen.Width = 2;
                DrawVertex(myDoc.VertexAt(i).X, myDoc.VertexAt(i).Y, myPen.Color, i);
            }
            for (int i = 0; i < MAXVERTICES; i++) //draw edges
            {
                for (int j = 0; j < MAXVERTICES; j++)
                {
                    if (myDoc.AdjMatrixWeight(i, j) != int.MaxValue)
                    {
                        myPen.Width = edgeWidth;
                        myPen.Color = myDoc.AdjMatrixColor(i, j);
                        DrawEdge(myPen, myDoc.VertexAt(i), myDoc.VertexAt(j));
                    }
                }
            }
            myPen.Color = defaultColor; //just in case
            for (int i = 0; i < MAXVERTICES; i++) //draw edge weights
            {
                for (int j = 0; j < MAXVERTICES; j++)
                {
                    if (myDoc.AdjMatrixWeight(i, j) != int.MaxValue)
                    {
                        DrawEdgeWeight(myPen, myDoc.VertexAt(i), myDoc.VertexAt(j));
                    }
                }
            }
            Invalidate();
        }
        
 //mouse events

        //When the image is clicked by the mouse, it checks which radio button is selected and performs to corresponding action
        private void DrawGraph(object sender, MouseEventArgs e)
        {
            if (!runAlgorithm) //disabled when algorithm is running
            {
                //sets graph back to default color if algorithm was not cleared
                if (algorithmOutput.Items.Count > 0) //true if algorithm was not cleared
                {
                    for (int i = 0; i < myDoc.VertexCount; i++)
                    {
                        for (int j = 0; j < myDoc.VertexCount; j++)
                        {
                            myDoc.createColorMatrix(i, j, defaultColor);
                        }
                    }
                }
                nextStepbtn.Enabled = false;
                //Make edges into arrows if directed
                if (directed)
                    myPen.EndCap = System.Drawing.Drawing2D.LineCap.ArrowAnchor; //https://msdn.microsoft.com/en-us/library/2bdt6zkb(v=vs.110).aspx
                if (addVertexrb.Checked == true && myDoc.VertexCount < MAXVERTICES)        //Add a new vertex
                {
                    if (e.X < 125 || e.Y < 60 || e.X > algorithmOutput.Location.X - VERTEXSIZE/2 || e.Y > ClientRectangle.Height - VERTEXSIZE/2) //too close to edge
                        return;
                    if (myDoc.NearVertex(e.X, e.Y) != new Point(-1, -1)) //Trying to draw a vertex on top of an existing vertex
                        return;
                    DrawVertex(e.X, e.Y, defaultColor, myDoc.VertexCount); //Draw vertev
                    if (AddPointEvent != null)
                        AddPointEvent(this, e);
                }
                if (addEdgerb.Checked == true && myDoc.VertexCount > 1)     //Add a new edge
                {
                    if (first != new Point(-1, -1) && second != new Point(-1, -1) && first != second) //location of mouse down and mouse up are at vertices and not the same vertex
                    {
                        if (myDoc.AdjMatrixWeight(myDoc.VertexNumber(first.X, first.Y), myDoc.VertexNumber(second.X, second.Y)) != int.MaxValue) //Edge already exists
                        {
                            return;
                        }
                        myPen.Width = edgeWidth;
                        myDoc.createAdjMatrix(myDoc.VertexNumber(first.X, first.Y), myDoc.VertexNumber(second.X, second.Y), weight); //add edge to adjacency matrix
                        myDoc.createColorMatrix(myDoc.VertexNumber(first.X, first.Y), myDoc.VertexNumber(second.X, second.Y), myPen.Color); //change color in color matrix
                        if (!directed) //edge goes both directions
                        {
                            myDoc.createAdjMatrix(myDoc.VertexNumber(second.X, second.Y), myDoc.VertexNumber(first.X, first.Y), weight);
                            myDoc.createColorMatrix(myDoc.VertexNumber(second.X, second.Y), myDoc.VertexNumber(first.X, first.Y), myPen.Color);
                        }
                        DrawEdge(myPen, first, second); //Draw edge
                        if (AddEdgeEvent != null)
                            AddEdgeEvent(this, e);
                    }

                }
                if (moveVertexrb.Checked == true && myDoc.VertexCount > 0)      //Move vertex from mouse down position to mouse up position
                {
                    int num = myDoc.VertexNumber(first.X, first.Y); //Find index of vertex clicked on (mouse down)
                    if (e.X < 125 || e.Y < 60 || e.X > algorithmOutput.Location.X - VERTEXSIZE / 2 || e.Y > ClientRectangle.Height - VERTEXSIZE / 2) //too close to edge
                        return;
                    if (num != -1) //there was a vertex where mouse down happened
                    {
                        if (MoveVertexEvent != null)
                            MoveVertexEvent(this, e);
                    }
                }
                if (deleteVertexrb.Checked == true && myDoc.VertexCount > 0)        //Delete vertex
                {
                    if (myDoc.NearVertex(e.X, e.Y) == new Point(-1, -1)) //did not click on a vertex
                        return;
                    if (DeleteVertexEvent != null)
                        DeleteVertexEvent(this, e);
                }
                if (deleteEdgerb.Checked == true && myDoc.VertexCount > 1)      //Delete edge
                {
                    for (int i = 0; i < myDoc.VertexCount; i++)
                    {
                        for (int j = 0; j < myDoc.VertexCount; j++)
                        {
                            if (myDoc.AdjMatrixWeight(i, j) != int.MaxValue)    //An edge exists from i to j
                            {
                                Point f = myDoc.VertexAt(i); //i's location
                                Point s = myDoc.VertexAt(j); //j's location
                                //decimal so slope stays decimal and is accurate enough
                                decimal x = Convert.ToDecimal(e.X); //mouse x
                                decimal y = Convert.ToDecimal(e.Y); //mouse y
                                decimal aX = Convert.ToDecimal(f.X); //first point's x
                                decimal aY = Convert.ToDecimal(f.Y); //first point's y
                                decimal bX = Convert.ToDecimal(s.X); //second point's x
                                decimal bY = Convert.ToDecimal(s.Y); //second point's y
                                decimal m = ((bY - aY) / (bX - aX)); //slope between points
                                // y = m(x-x1) + y1 - equation of line
                                if ((y < (m * (x - aX) + aY + Convert.ToDecimal(5))) && (y > (m * (x - aX) + aY - Convert.ToDecimal(5)))) //5 pixel leeway in every direction
                                {
                                    //both directions because distinguishing between the two for which one the user wants by mouse click is impossible
                                    myDoc.createAdjMatrix(i, j, int.MaxValue); //Clear edge from i to j out of adjacency matrix
                                    myDoc.createAdjMatrix(j, i, int.MaxValue); //Clear edge from j to i out of adjacency matrix
                                    myDoc.createColorMatrix(i, j, Color.Empty); //Clear color out of color matrix
                                    myDoc.createColorMatrix(j, i, Color.Empty); //Clear color out of color matrix
                                    if (AddEdgeEvent != null)
                                        AddEdgeEvent(this, e);
                                    return;
                                }
                            }
                        }
                    }

                }
            }

        }

        //Sets first to equal the location of the mouse when mouse down happened
        // and mDown to equal true - if algorithm is not running
        private void mouseDown(object sender, MouseEventArgs e)
        {
            if (!runAlgorithm)
            {
                mDown = true;
                first = myDoc.NearVertex(e.X, e.Y);
                if (MouseDownEvent != null)
                    MouseDownEvent(this, e);
            }
        }

        //Sets mDown to false and redraws the graph
        // - If algorithm is not running
        private void mouseUp(object sender, MouseEventArgs e)
        {
            if (!runAlgorithm)
            {
                mDown = false;
                RedrawGraph();
            }
        }

        //If an algorithm is not running,
        //sets second to equal the location of any vertex the  
        //current mouse location may be near/on or it equals  
        //(-1, -1) if not near a vertex, sets mlocation to equal 
        //the current mouse location, and if certain conditions  
        //are met, draws an edge as it is being dragged
        private void mouseMove(object sender, MouseEventArgs e)
        {
            if (!runAlgorithm)
            {
                //location of any vertex the current mouse location may 
                //be near/on or it equals (-1, -1) if not near a vertex
                second = myDoc.NearVertex(e.X, e.Y); 
                mlocation = new Point(e.X, e.Y); //current location of mouse
                if (mDown && addEdgerb.Checked == true && myDoc.VertexCount > 1)
                {
                    //Draws an edge as it is being dragged
                    RedrawGraph();
                    myPen.Color = Color.Gray;
                    DrawEdge(myPen, first, mlocation); //draws dragged edge in gray
                    myPen.Color = defaultColor;
                }
            }
        } 

    //Graph Options
        //Set everything to default values and redraw to clear screen
        private void onClearGraphClick(object sender, EventArgs e)
        {
            if (!runAlgorithm)
            {
                if (ClearGraphEvent != null)
                    ClearGraphEvent(this, e);
                RedrawGraph();

            }
        }
        //When clicked, if it says directed, the graph will become non-directed
        // and edges that only went one way between edges will now go both ways.
        // If it says Non-directed, the graph will become directed.
        // Either way, the graph is redraw.
        private void isDirectedClick(object sender, EventArgs e)
        {
            if (directed)
            {
                for (int i = 0; i < myDoc.VertexCount; i++)
                {
                    for (int j = 0; j < myDoc.VertexCount; j++)
                    {
                        if (myDoc.AdjMatrixWeight(i, j) != int.MaxValue)    //An edge exists between i and j
                        {
                            DialogResult result = MessageBox.Show("All edges will become connected in both directions!", 
                                "WARNING!", MessageBoxButtons.OKCancel);
                            if (result == DialogResult.OK)
                            {
                                directed = false;
                                directedtsmi.Text = "Non-Directed";
                                //Make it so the edge weight between k and l is also the edge weight between l and k
                                for (int k = 0; k < MAXVERTICES; k++)
                                {
                                    for (int l = 0; l < MAXVERTICES; l++)
                                    {
                                        if (myDoc.AdjMatrixWeight(k, l) != int.MaxValue)
                                        {
                                            myDoc.createAdjMatrix(l, k, myDoc.AdjMatrixWeight(k, l)); 
                                        }
                                    }
                                }
                                myPen.EndCap = System.Drawing.Drawing2D.LineCap.Flat; //Make edges normal lines instead of arrows
                                RedrawGraph();
                                return;
                            }
                        }
                    }
                }
                directed = false;
                directedtsmi.Text = "Non-Directed";
                myPen.EndCap = System.Drawing.Drawing2D.LineCap.Flat; //Make edges normal lines instead of arrows
            }
            else
            {
                directed = true;
                directedtsmi.Text = "Directed";
                myPen.EndCap = System.Drawing.Drawing2D.LineCap.ArrowAnchor; //Make edges arrows instead of normal lines
                RedrawGraph();
            }
        }

        //Opens a dialog box for the user to choose an edge weight
        private void onWeightClick(object sender, EventArgs e)
        {
            frmChooseWeight cdlg = new frmChooseWeight();
            cdlg.ShowDialog();
            if (cdlg.DialogResult == DialogResult.OK)
            {
                weight = cdlg.Weight;
                weighttsmi.Text = "Weight: " + weight; //show current weight in the menu
            }
        }

        

//FillMRU, and everything dealing with opening, saving, new file, update caption, and closing
    //are from CSCI 350 Lab 6 and belong to Dr. McVey
        private void FillMRU()
        {
            MRU = new List<string>();
            try
            {
                using (StreamReader sr = new StreamReader(mruFilePath))
                {
                    string num = sr.ReadLine();
                    int count = Convert.ToInt32(num);
                    for (int i = 0; i < count; i++)
                    {
                        string x = sr.ReadLine();
                        if (!MRU.Contains(x))
                        {
                            MRU.Insert(0, x);
                        }
                    }
                    sr.Close();
                }
            }
            catch (Exception e)
            {
                MessageBox.Show("Can't open file: " + mruFilePath + "\r\n" + e.Message);
                return;
            }
            //MRU.Reverse(0, MRU.Count);
            foreach (string s in MRU)
            {
                if (File.Exists(s))
                {
                    ToolStripMenuItem tsmi = new ToolStripMenuItem(s);
                    tsmi.Tag = s;
                    tsmi.Click += OpenRecentFile;
                    openRecentToolStripMenuItem.DropDownItems.Add(tsmi);
                }
            }
        }

        private void OpenRecentFile(object sender, EventArgs e)
        {
            myDoc.Open(sender.ToString());
        }

        public void Open(string filename)
        {
            myDoc.Open(filename);
        }
//New File
        private void onNewFile(object sender, EventArgs e)
        {
            myDoc.NewFile();
        }
        private void viewNewDocument(object sender, EventArgs e)
        {
            img = new Bitmap(ClientRectangle.Width, ClientRectangle.Height);
            imgGraphics = Graphics.FromImage(img);
            Invalidate();
        }
//Open File
        private void onOpenFileClick(object sender, EventArgs e)
        {
            myDoc.Open();
        }
        private void viewOpenDocument(object sender, EventArgs e)
        {
            // update img
            img = new Bitmap(ClientRectangle.Width, ClientRectangle.Height);
            imgGraphics = Graphics.FromImage(img);

            if (directed)
                myPen.EndCap = System.Drawing.Drawing2D.LineCap.ArrowAnchor;
            RedrawGraph();

            string s = myDoc.FileName;
            ToolStripMenuItem tsmi = new ToolStripMenuItem(s);
            if (!MRU.Contains(s))
            {
                //MRU.Add(s);
                MRU.Insert(0, s);
            }
            else
            {
                MRU.Remove(s);
                //openRecentToolStripMenuItem.DropDownItems.Remove(tsmi);
                int index = 0;
                int found = 0;
                foreach (ToolStripMenuItem TSMI in openRecentToolStripMenuItem.DropDownItems)
                {
                    if (TSMI.Text == tsmi.Text)
                    {
                        found = index;
                    }
                    index++;
                }
                openRecentToolStripMenuItem.DropDownItems.RemoveAt(found);
                MRU.Insert(0, s);
            }
            //ToolStripMenuItem tsmi = new ToolStripMenuItem(s);   //  tsmi.Tag = s;
            tsmi.Click += OpenRecentFile;
            openRecentToolStripMenuItem.DropDownItems.Insert(0, tsmi);
            Invalidate();
        }
//Save options
        private void onSaveClick(object sender, EventArgs e)
        {
            myDoc.Save();
            string s = myDoc.FileName;
            ToolStripMenuItem tsmi = new ToolStripMenuItem(s);   //  tsmi.Tag = s;
            if (!MRU.Contains(s))
            {
                //MRU.Add(s);
                MRU.Insert(0, s);
            }
            else
            {
                MRU.Remove(s);
                int index = 0;
                int found = 0;
                foreach (ToolStripMenuItem TSMI in openRecentToolStripMenuItem.DropDownItems)
                {
                    if (TSMI.Text == tsmi.Text)
                    {
                        found = index;
                    }
                    index++;
                }
                openRecentToolStripMenuItem.DropDownItems.RemoveAt(found);
                MRU.Insert(0, s);
            }
            tsmi.Click += OpenRecentFile;
            openRecentToolStripMenuItem.DropDownItems.Add(tsmi);
            Invalidate();
        }

        private void onSaveAsClick(object sender, EventArgs e)
        {
            myDoc.SaveAs();
            string s = myDoc.FileName;
            ToolStripMenuItem tsmi = new ToolStripMenuItem(s);   //  tsmi.Tag = s;
            if (!MRU.Contains(s))
            {
                //MRU.Add(s);
                MRU.Insert(0, s);
            }
            else
            {
                MRU.Remove(s);
                int index = 0;
                int found = 0;
                foreach (ToolStripMenuItem TSMI in openRecentToolStripMenuItem.DropDownItems)
                {
                    if (TSMI.Text == tsmi.Text)
                    {
                        found = index;
                    }
                    index++;
                }
                openRecentToolStripMenuItem.DropDownItems.RemoveAt(found);
                MRU.Insert(0, s);
            }
            tsmi.Click += OpenRecentFile;
            openRecentToolStripMenuItem.DropDownItems.Add(tsmi);
            Invalidate();
        }

        /*** Update Caption **  Based on Sells */
        private void viewUpdateCaption(object sender, EventArgs e)
        {
            if (myDoc.FileName.Length > 20)
            {
                int pos = myDoc.FileName.LastIndexOf('\\');
                string x = myDoc.FileName.Substring(pos);
                this.Text = myDoc.FileName.Substring(0, 20 - x.Length - 3)
                                    + "..." + x;
            }
            else if (string.IsNullOrEmpty(myDoc.FileName))
                this.Text = "Untitled";
            else
                this.Text = myDoc.FileName;
            if (myDoc.HasBeenModified)
                this.Text += "*";
        }

        private void menuExitClick(object sender, EventArgs e)
        {
            this.Close();
        }

        private void frmClosing(object sender, FormClosingEventArgs e)
        {
            DialogResult d = myDoc.Close();
            if (d == DialogResult.Yes)
            {
                if (!myDoc.SaveAs()) // or Save();
                    e.Cancel = true;
            }
            else if (d == DialogResult.Cancel)
                e.Cancel = true;
        }

        private void frmMainFormClosed(object sender, FormClosedEventArgs e)
        {
            try
            {
                MRU.Reverse();
                using (StreamWriter sw = new StreamWriter(mruFilePath))
                {
                    sw.WriteLine(MRU.Count.ToString());
                    for (int i = 0; i < MRU.Count; i++)
                        sw.WriteLine(MRU[i]);
                    sw.Close();
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show("Can't save file: " + mruFilePath
                                        + "\r\n" + ex.Message, Application.ProductName);
                return;
            }
        }


  //Algorithms

        //Opens dialog box where user selects which algorithm they would like to run
        private void onAlogithmsClick(object sender, EventArgs e)
        {
            onClearAlgorithmClick(sender, e); //Make sure algorithm is cleared
            frmChooseAlgorithm cdlg = new frmChooseAlgorithm();
            cdlg.Count = myDoc.VertexCount; //send choose algorithm form the vertex count
            cdlg.ShowDialog(); //show the choose algorithm form
            if (cdlg.DialogResult == DialogResult.OK)
            {
                algor = cdlg.Algorithm; //get chosen algorithm from form
                start = cdlg.StartVertex; //get start vertex from form
            }
            runAlgorithmbtn.Text = "Run Algorithm";
            //Enable buttons that run the algorithm
            runAlgorithmbtn.Enabled = true;
            nextStepbtn.Enabled = true;
        }

        //Adds new vertex color to vertexColor array and new edge color to the color matrix
        //or changes that color in array/matrix if going backwards
        private void RedrawPartOfGraph( int pv, int v, MyDocument.VertexState state, string edge)
        {
            //Find color of vertex
            Color c = defaultColor;
            if (state == MyDocument.VertexState.Visited)
                c = visitedColor;
            else if (state == MyDocument.VertexState.Done)
                c = doneColor;
            myDoc.vertexColor[v] = c; //add new color of vertex to vertex color array
            //Find color that matches edge type
            c = defaultColor;
            if (edge == myDoc.treeEdge)
                c = treeEdgeColor;
            else if (edge == myDoc.backEdge)
                c = backEdgeColor;
            else if (edge == myDoc.forwardEdge)
                c = forwardEdgeColor;
            else if (edge == myDoc.crossEdge)
                c = crossEdgeColor;
            else if (edge == myDoc.tempEdge)
                c = tempEdgeColor;

            pv = curValue.parent; //index of parent vertex
            if (pv != -1) //a parent exists
            {
                if (myDoc.AdjMatrixWeight(pv, v) != int.MaxValue)
                    myDoc.createColorMatrix(pv, v, c); //add new color to color matrix
            }

        }

        //Redraws graph with algorithm colors
        private void RedrawGraph2()
        {
            imgGraphics.Clear(Color.Transparent); //Clear image/graph
            //find respective color and draw vertex
            for (int i = 0; i < myDoc.VertexCount; i++)
            {
                Color c = myDoc.vertexColor[i];
                if (c != visitedColor && c != doneColor)
                {
                    c = defaultColor;
                    myPen.Width = 2;
                }
                else
                    myPen.Width = 4;
                DrawVertex(myDoc.VertexAt(i).X, myDoc.VertexAt(i).Y, c, i);
            }
            if (directed) //make edges arrows if directed
                myPen.EndCap = System.Drawing.Drawing2D.LineCap.ArrowAnchor;
            //find respective color and draw edge
            for (int i = 0; i < MAXVERTICES; i++)
            {
                for (int j = 0; j < MAXVERTICES; j++)
                {
                    if (myDoc.AdjMatrixWeight(i, j) != int.MaxValue)
                    {
                        myPen.Color = myDoc.AdjMatrixColor(i, j);
                        myPen.Width = edgeWidth;
                        DrawEdge(myPen, myDoc.VertexAt(i), myDoc.VertexAt(j));
                    }
                }
            }
            myPen.Color = defaultColor;
            //Draw edge weights
            for (int i = 0; i < MAXVERTICES; i++)
            {
                for (int j = 0; j < MAXVERTICES; j++)
                {
                    if (myDoc.AdjMatrixWeight(i, j) != int.MaxValue)
                        DrawEdgeWeight(myPen, myDoc.VertexAt(i), myDoc.VertexAt(j));
                }
            }
            myPen.Width = 2;
            myPen.Color = defaultColor;
            Invalidate();
        }

        //Performs one step of an algorithm, forwards if backwards is false, backwards if backwards is true
        private void StepofAlgorithm(MyDocument.StepResult currentValue, bool backwards = false)
        {
            if (!backwards) //Next Step
            {
                //clear temporary edges when tree edges are added
                foreach (var edge in currentValue.otherEdges)
                {
                    myDoc.createColorMatrix(edge.start, edge.end, defaultColor);
                }
                string s = currentValue.output;
                algorithmOutput.Items.Add(s); //Add step text output to algorithm output textbox
                RedrawPartOfGraph(currentValue.parent, currentValue.vertex, currentValue.state, currentValue.edgeType); //change colors in vertex array and color matrix (edges)
            }
            else //Step Back
            {
                //show temporary edges that may have been cleared
                foreach (var edge in currentValue.otherEdges)
                {
                    myDoc.createColorMatrix(edge.start, edge.end, tempEdgeColor);
                }
                //Use previous state and edge type to change colors of graph back
                RedrawPartOfGraph(currentValue.parent, currentValue.vertex, currentValue.prevState, currentValue.prevEdgeType);
            }
           
            RedrawGraph2(); //redraw graph with color changes
        }

        //Sets everything back to the way it was before an algorithm was ran
        private void onClearAlgorithmClick(object sender, EventArgs e)
        {
            runAlgorithm = false;
            //Set values in color matrix back to the default color (black)
            for (int i = 0; i < myDoc.VertexCount; i++)
            {
                for (int j = 0; j < myDoc.VertexCount; j++)
                {
                    myDoc.createColorMatrix(i, j, defaultColor);
                }
            }
            myPen.Width = 2;
            myPen.Color = defaultColor;
            RedrawGraph(); //Redraw the graph in the default color (black)
            algorithmOutput.Items.Clear(); //clear the algorithm output textbox
            step = 0; //set algorithm step number to 0
            //Reset buttons
            nextStepbtn.Text = "Step Through";
            stepBackbtn.Text = "-";
            if (ClearAlgorithmEvent != null)
                ClearAlgorithmEvent(this, e);
        }

        //runs an algorithm to a timer click
        private void onRunAlgorithmClick(object sender, EventArgs e)
        {
            onClearAlgorithmClick(sender, e);
            //select an algorithm
            if (algor == "BFS")
                enumer = myDoc.BFS(start).GetEnumerator();
            else if (algor == "DFS")
                enumer = myDoc.DFS(start).GetEnumerator();
            else if (algor == "MST")
                enumer = myDoc.MST(start).GetEnumerator();
            else if (algor == "BFM")
                enumer = myDoc.BFM(start).GetEnumerator();

            timer1.Enabled = true; //start timer
            runAlgorithm = true; //algorithm is running - bool
            runAlgorithmbtn.Text = "Running";
            //Disable most buttons so data is not corrupted
            runAlgorithmbtn.Enabled = false;
            clearGraphbtn.Enabled = false;
            clearAlgorithmbtn.Enabled = false;
            directedtsmi.Enabled = false;
            nextStepbtn.Enabled = false;
            //enable stop algorithm button so algorithm can be stopped
            stopAlgorithmbtn.Enabled = true;
            pausePlaybtn.Enabled = true; //so algorithm can be paused
        }

        //Pauses the animation if clicked when it says pause.
        //If it says continue, it continues the animation from 
        //where it was paused
        private void onPausePlay(object sender, EventArgs e)
        {
            if (pausePlaybtn.Text == "Pause") //pause animation
            {
                timer1.Enabled = false; //stop timer
                pausePlaybtn.Text = "Continue";
            }
            else
            {
                timer1.Enabled = true; //start timer again
                pausePlaybtn.Text = "Pause";
            }
        }

        //Resets everything to what it was before the animation ran
        private void onStopAlgorithm(object sender, EventArgs e)
        {
            timer1.Enabled = false; //stop timer
            runAlgorithm = false;
            runAlgorithmbtn.Text = "Run Algorithm"; //change text of run algorithm button
            clearGraphbtn.Enabled = true; //enable clear graph button
            clearAlgorithmbtn.Enabled = true; // enable clear algorithm button
            directedtsmi.Enabled = true; //enable ability to switch between directed and non-directed
            pausePlaybtn.Enabled = false; //disable the pause button
            pausePlaybtn.Text = "Pause"; //make sure the pause button says pause not continue
            stopAlgorithmbtn.Enabled = false; //disable stop algorithm button - no algorithm running to stop
        }

        //Changes the length between ticks of the timer depending on value of the track bar
        private void onTrackBarValueChange(object sender, EventArgs e)
        {
            timer1.Interval = trackBar1.Value * 500 + 500;
        }

        //If button says "End - Clear", it resets everything back 
        //to what it was before the algorithm ran.
        //If button says "Step Through", it starts the algorithm.
        //Else it goes forward one step in the algorithm.
        private void onNextStepClick(object sender, EventArgs e)
        {
            if (nextStepbtn.Text == "End - Clear")
            {
                runAlgorithm = false;
                clearGraphbtn.Enabled = true; //enable clear graph button
                directedtsmi.Enabled = true; //enable ability to switch between directed and non-directed
                runAlgorithmbtn.Enabled = true; //enable run algorithm button
                onClearAlgorithmClick(sender, e); //Clear algorithm
                return;
            }
            stepBackbtn.Text = "Step Back";
            if (nextStepbtn.Text == "Step Through")
            {
                onClearAlgorithmClick(sender, e); //make sure algorithm is cleared - so data is not corrupted
                runAlgorithm = true;
                clearGraphbtn.Enabled = false; //disable clear graph button
                directedtsmi.Enabled = false; //disable ability to switch between directed and non-directed
                runAlgorithmbtn.Enabled = false; //disable run algorithm button
                nextStepbtn.Text = "Next Step";
                step = 0; //current step number is 0 - just starting
                //select an algorithm
                if (algor == "BFS")
                    enumer = myDoc.BFS(start).GetEnumerator();
                else if (algor == "DFS")
                    enumer = myDoc.DFS(start).GetEnumerator();
                else if (algor == "MST")
                    enumer = myDoc.MST(start).GetEnumerator();
                else if (algor == "BFM")
                    enumer = myDoc.BFM(start).GetEnumerator();
                stepList = new List<MyDocument.StepResult>(); //list of processed steps
            }

            if (step == stepList.Count) //current step number is equal to number of total steps processed
            {
                if (enumer.MoveNext()) //another step in algorithm exists
                {
                    curValue = enumer.Current; //current step result
                    stepList.Add(curValue); //add current step result to the list
                    StepofAlgorithm(curValue); //perform a step of the algorithm
                    step++; //increment step number
                }
                else //algorithm is done
                    nextStepbtn.Text = "End - Clear"; 
            }
            else //current step number is less than the number of total steps processed
            {
                curValue = stepList[step]; //get step result from list of previous step results
                StepofAlgorithm(curValue); //perform a step of the algorithm
                step++; //increment step number
            }
        }

        //Goes back a step in the algorithm
        private void onStepBackClick(object sender, EventArgs e)
        {
            if (step > 0)
                stepBackbtn.Text = "Step Back";
            if (stepBackbtn.Text == "Step Back") //only true when stepping through an algorithm and after first step
            {
                step = algorithmOutput.Items.Count - 1; //make sure step number matches what is in the algorithm output text
                algorithmOutput.Items.RemoveAt(step); //remove last item in the algorithm output text
                curValue = stepList[step]; //find current step result using the list
                if (step > 0)
                {
                    StepofAlgorithm(curValue, true); //go through step of algorithm with previous state and edge type
                }
                else
                {
                    RedrawGraph();
                    stepBackbtn.Text = "-";
                }
                if (nextStepbtn.Text == "End - Clear") //If step back happens on last step
                {
                    nextStepbtn.Text = "Next Step";
                }
            }
        }

        //run a step of the algorithm every timer click
        private void onTimerClick(object sender, EventArgs e)
        {
            if(runAlgorithm)
            {
                if (enumer.MoveNext()) //if another step of the algorithm exists
                {
                    curValue = enumer.Current; //get current step result values
                    StepofAlgorithm(curValue); //perform one step of the algorithm
                    
                }
                else //algorithm is done
                {
                    timer1.Enabled = false; //stop timer
                    //set enabled/disabled back to what it was before algorithm was ran
                    runAlgorithm = false;
                    runAlgorithmbtn.Text = "Run Algorithm";
                    runAlgorithmbtn.Enabled = true;
                    clearGraphbtn.Enabled = true;
                    clearAlgorithmbtn.Enabled = true;
                    directedtsmi.Enabled = true;
                    nextStepbtn.Enabled = true;
                    stopAlgorithmbtn.Enabled = false;
                    pausePlaybtn.Enabled = false;
                }
            }
        }
  
    }
}
