﻿//Some functions are from CSCI 350 Lab 6 and belong to Dr. McVey (see comments below)
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.Win32; // needed for registry?
using System.ComponentModel;
using System.Data;
using System.Collections;

namespace Senior_Capstone
{
    class MyDocument
    {
        //Graph variables:
        const int MAXVERTICES = 30;
        const int VERTEXSIZE = 25;
        private Point[] myVertices;
        private int Count;
        private Point first;
        private int[,] adjMatrix;
        private Color[,] colorAdjMatrix;
        public Color[] vertexColor;
        
        //Algorithm variables:
        public enum VertexState { Unvisited, Visited, Done };
        private int[] VertexStates;
        private int parentVertex = -1;  //BFS and MST
        private Stack<int> parent;  //DFS only
        private int[] startTimes;   //DFS only
        private int time = 0;       //DFS only
        private int[] distance;      //Shortest Path
        private int[] pred;         //Shortest Path
        //Edge Types:
        public string initialEdge = "InitialEdge";
        public string treeEdge = "TreeEdge";
        public string backEdge = "BackEdge";
        public string forwardEdge = "ForwardEdge";
        public string crossEdge = "CrossEdge";
        public string tempEdge = "TemporaryEdge"; //an edge that is not one of the other types and temporary
        

        // "any document" data
        private bool hasBeenModified = false;
        private string fileName = "";
        private Form host;

        // the added 'nice' stuff - from Lab 6, only changed names
        private string defaultFileName = "Untitled";
        private string fileExtension = "cap";
        private string fileExtensionDesc = "graph algorithms";
        private bool registerFileExtension = true;
        private string filter = "CAP Files (*.cap) |*.cap";
        private string iconResourceIndex = "bmvIcon2.ico";

        /* EVENTS */
        public event EventHandler NewDocument;
        public event EventHandler OpenDocument;
        public event EventHandler UpdateCaption;


        public MyDocument()
        {
            myVertices = new Point[MAXVERTICES];
            adjMatrix = new int[MAXVERTICES, MAXVERTICES];
            colorAdjMatrix = new Color[MAXVERTICES, MAXVERTICES];
            vertexColor = new Color[MAXVERTICES];
            VertexStates = new int[MAXVERTICES];
            parent = new Stack<int>();
            parent.Push(-1);
            startTimes = new int[MAXVERTICES];
            distance = new int[MAXVERTICES];
            pred = new int[MAXVERTICES];
            for (int i = 0; i < MAXVERTICES; i++)
            {
                for (int j = 0; j < MAXVERTICES; j++)
                {
                    adjMatrix[i, j] = int.MaxValue;
                }
                distance[i] = int.MaxValue;
                pred[i] = -1;
            }
            hasBeenModified = false;
            if (this.UpdateCaption != null)
                this.UpdateCaption(this, EventArgs.Empty);

        }

        /* PROPERTIES  - generic document */
        //-From Lab 6
        public bool RegisterFileExtensionWithShell  /* SELLS/MCVEY */
        {
            get { return this.registerFileExtension; }
            set { this.registerFileExtension = value; }
        }

        public bool HasBeenModified
        {
            set { hasBeenModified = value; }
            get { return hasBeenModified; }
        }
        public string FileName
        {
            set { fileName = value; }
            get { return fileName; }
        }
        public Form Host
        {
            set { host = value; }
            get { return host; }
        }
        /* PROPERTIES  -  specific to this document */
        //Current number of vertices
        public int VertexCount 
        {
            set { Count = value; }
            get { return Count; }
        }
        //Takes vertex number and returns location of vertex
        public Point VertexAt(int index) 
        {
            return myVertices[index];
        }
        //Takes a point a returns the location of the vertex closest to the given point
        // or if no vertex is close by, returns (-1, -1)
        public Point NearVertex(int x, int y)  
        {
            for (int i = 0; i < VertexCount; i++)
            {
                int vx = myVertices[i].X;
                int vy = myVertices[i].Y;
                if ((x > (vx - VERTEXSIZE) && x < (vx + VERTEXSIZE)) && (y > (vy - VERTEXSIZE) && y < (vy + VERTEXSIZE)))
                    return new Point(vx, vy);
            }
            return new Point(-1, -1);
        }
        //Takes a point and returns the vertex index corresponding to that point
        public int VertexNumber(int x, int y)
        {
            for (int i = 0; i < VertexCount; i++)
            {
                int vx = myVertices[i].X;
                int vy = myVertices[i].Y;
                if (vx == x && vy == y)
                    return i;
            }
            return -1;
        }        
        //This fuction takes two vertices, a and b, and the weight of the edge 
        // between them from a to b, and puts that weight into the ath row and 
        // bth column of the adjacency matrix
        public void createAdjMatrix(int a, int b, int weight)
        {
            adjMatrix[a, b] = weight; 
        }
        //This function takes two vertices and returns the weight of the 
        //edge between them going from a to b
        public int AdjMatrixWeight(int a, int b)
        {
            return adjMatrix[a, b];
        }
        //This fuction takes two vertices, a and b, and the color of the edge 
        // between them, and puts that color into the ath row and bth column
        // of the adjacency matrix
        public void createColorMatrix(int a, int b, Color c)
        {
            colorAdjMatrix[a, b] = c;
        }
        //This function takes two vertices and returns the color of the 
        //edge between them 
        public Color AdjMatrixColor(int a, int b)
        {
            return colorAdjMatrix[a, b];
        }


        /* METHODS */

        //If mouse click is near a vertex, the function returns without doing anything
        //Else if the vertex count is less MAXVERTICES, it makes a vertex where the mouse click happened
        public void docMouseClick(object sender, MouseEventArgs e) //Based off of function from Lab 6
        {
            if (NearVertex(e.X, e.Y) != new Point(-1, -1)) //mouse click is near a vertex
                return;
            if (VertexCount < MAXVERTICES)
            {
                myVertices[VertexCount] = new Point(e.X, e.Y); //add location of mouse click to array of vertices
                VertexStates[VertexCount] = (int)VertexState.Unvisited; //mark new vertex as unvisited
                vertexColor[VertexCount] = Color.Black; //set the color of the new vertex to black
                VertexCount++; //increment vertex count
                hasBeenModified = true;
                if (this.UpdateCaption != null)
                    this.UpdateCaption(this, EventArgs.Empty);
            }
        }

        //Updates hasBeenModified and UpdateCaption when an edge has been added
        public void docAddEdge(object sender, MouseEventArgs e)
        {
            hasBeenModified = true;
            if (this.UpdateCaption != null)
                this.UpdateCaption(this, EventArgs.Empty);
        }

        //Gets the location of the mouse down to use in docVertexMove
        public void docMouseDown(object sender, MouseEventArgs e)
        {
            first = new Point(e.X, e.Y);
        }

        //Changes location of vertex when mouse it moved
        public void docVertexMove(object sender, MouseEventArgs e)
        {
            Point b = NearVertex(first.X, first.Y); //get vertex closest to mouse down
            int num = VertexNumber(b.X, b.Y); //get that vertex's index
            Point c = new Point(e.X, e.Y); 
            if (NearVertex(c.X, c.Y) == new Point(-1, -1)) //mouse location is not near a vertex
            {
                myVertices[num] = c; //vertex's location is reset to mouse location
                hasBeenModified = true;
                if (this.UpdateCaption != null)
                    this.UpdateCaption(this, EventArgs.Empty);
            }
        }

        //Resets array[vertex number] to their default values
        public void docDeleteVertex(object sender, MouseEventArgs e)
        {
            Point a = NearVertex(e.X, e.Y); //get location of vertex near mouse click
            int num = VertexNumber(a.X, a.Y); //get index of that vertex
            //reset edge weights back to max int
            if (num != VertexCount)
            {
                for (int j = 0; j < (VertexCount - num); j++)
                {
                    for (int i = num; i < VertexCount; i++)
                    {
                        adjMatrix[i, num + j] = adjMatrix[i, num + 1 + j];
                        adjMatrix[num + j, i] = adjMatrix[num + 1 + j, i];
                        if (i == num + j)
                            adjMatrix[num + j, i] = int.MaxValue;
                    }
                }
                for (int i = num; i < VertexCount; i++)
                {
                    myVertices[i] = myVertices[i + 1];
                }
            }
            for (int i = 0; i < VertexCount; i++)
            {
                adjMatrix[i, VertexCount - 1] = int.MaxValue;
                adjMatrix[VertexCount - 1, i] = int.MaxValue;
            }
            vertexColor[VertexCount] = Color.Empty; //reset color back to empty (default color in initialization)
            VertexCount--; //decrement vertex count
            hasBeenModified = true;
            if (this.UpdateCaption != null)
                this.UpdateCaption(this, EventArgs.Empty);
        }

        //Resets everything to default values
        public void docClearGraph(object sender, EventArgs e)
        {
            for (int i = 0; i < VertexCount; i++)
            {
                for (int j = 0; j < VertexCount; j++)
                {
                    adjMatrix[i, j] = int.MaxValue;    //Clear adjMatrix of weights
                    colorAdjMatrix[i, j] = Color.Empty; //Clear color adjacency matrix
                }

            }
            if (VertexCount > 0)
            {
                hasBeenModified = true;
                if (this.UpdateCaption != null)
                    this.UpdateCaption(this, EventArgs.Empty);
            }
            VertexCount = 0; //set number of vertices to zero
        }

//Everything dealing with new, open, close, save, ect - From lab 6 
  //with minior changes mainly dealing with the data and some variable names
        /*** New document ***/
        public bool NewFile()
        {
            if (this.hasBeenModified)
            {
                DialogResult result = MessageBox.Show(fileName, "Do you wish to save?",
                            MessageBoxButtons.YesNoCancel);
                if (result == DialogResult.Yes)
                {
                    if (string.IsNullOrEmpty(fileName))
                    {
                        Save(true);
                    }
                    else
                    {
                        Save(false);
                    }
                }
                else if (result == DialogResult.Cancel)
                {
                    return true;
                }
            }
            for (int i = 0; i < VertexCount; i++)
            {
                for (int j = 0; j < VertexCount; j++)
                {
                    adjMatrix[i, j] = int.MaxValue;    //Clear adjMatrix
                    colorAdjMatrix[i, j] = Color.Empty;
                }
                VertexStates[i] = (int)VertexState.Unvisited;
            }
            hasBeenModified = false;
            VertexCount = 0;
            fileName = defaultFileName;

            // Let user know a "new" document has been "created"
            if (this.NewDocument != null)
                this.NewDocument(this, EventArgs.Empty);
            if (this.UpdateCaption != null)
                this.UpdateCaption(this, EventArgs.Empty);

            return true;
        }

        /*** Open existing file ***/

        public bool Open(string newFileName = "")
        {
            // Check if we can close current file
            // Get a file name

            if (this.hasBeenModified)
            {
                DialogResult result = MessageBox.Show(fileName, "Do you wish to save?",
                            MessageBoxButtons.YesNoCancel);
                if (result == DialogResult.Yes)
                {
                    if (string.IsNullOrEmpty(newFileName))
                    {
                        Save(true);
                    }
                    else
                    {
                        Save(false);
                    }
                }
                else if (result == DialogResult.Cancel)
                {
                    return true;
                }
            }


            if (string.IsNullOrEmpty(newFileName))
            {
                OpenFileDialog dlg = new System.Windows.Forms.OpenFileDialog();
                // Set up dialog
                dlg.DefaultExt = fileExtension;
                dlg.Filter = filter;

                // Get the file to open
                DialogResult res = dlg.ShowDialog();
                if (res != DialogResult.OK)
                    return false;
                
                newFileName = dlg.FileName;
            }
            //Make sure graph is cleared
            for (int i = 0; i < VertexCount; i++)
            {
                for (int j = 0; j < VertexCount; j++)
                {
                    adjMatrix[i, j] = int.MaxValue;    //Clear adjMatrix of weights
                    colorAdjMatrix[i, j] = Color.Empty; //Clear color adjacency matrix
                }
                VertexStates[i] = (int)VertexState.Unvisited;
            }
            VertexCount = 0; //set number of vertices to zero

            // Read the data
            try
            {

                using (StreamReader sr = new StreamReader(newFileName))
                {
                    string count = sr.ReadLine();
                    VertexCount = Convert.ToInt32(count);
                    for (int i = 0; i < VertexCount; i++)
                    {
                        string x = sr.ReadLine();
                        string[] a = x.Split(' ');
                        myVertices[i] = new Point(Convert.ToInt32(a[0]), Convert.ToInt32(a[1]));
                    }
                    for (int i = 0; i < VertexCount; i++)
                    {
                        string x = sr.ReadLine();
                        string[] a = x.Split(' ');
                        for (int j = 0; j < VertexCount; j++)
                        {
                            adjMatrix[i, j] = Convert.ToInt32(a[j]);
                            if (adjMatrix[i, j] != int.MaxValue)
                                createColorMatrix(i, j, Color.Black);
                        }
                    }
                }
            }
            catch (Exception e)
            {
                MessageBox.Show("Can't open file: " + newFileName + "\r\n" + e.Message, Application.ProductName);
                return false;
            }

            // Clear dirty bit, cache the file name and set the caption
            this.hasBeenModified = false;
            this.fileName = newFileName;

            // Success
            if (this.OpenDocument != null)
                this.OpenDocument(this, EventArgs.Empty);
            if (this.UpdateCaption != null)
                this.UpdateCaption(this, EventArgs.Empty);

            return true;
        }
        /*** Saving the data ***/
        public bool Save()
        {
            return Save(false);
        }

        public bool SaveAs()
        {
            return Save(true);
        }
        protected bool Save(bool type)
        {
            // Get the file name
            string newFileName = fileName;
            if (type || string.IsNullOrEmpty(newFileName))
            {
                // Set up the dialog
                SaveFileDialog dlg = new SaveFileDialog();
                dlg.DefaultExt = fileExtension;
                dlg.Filter = filter;
                if (!string.IsNullOrEmpty(newFileName))
                {
                    dlg.InitialDirectory = Path.GetDirectoryName(newFileName);
                    dlg.FileName = Path.GetFileName(newFileName);
                }
                else
                {
                    dlg.FileName = this.fileName;
                }


                DialogResult res = dlg.ShowDialog();
                if (res != DialogResult.OK) return false;
                newFileName = dlg.FileName;
            }

            // Write the data
            try
            {
                using (StreamWriter sw = new StreamWriter(newFileName))
                {
                    sw.WriteLine(VertexCount.ToString());
                    for (int i = 0; i < VertexCount; i++)
                        sw.WriteLine(myVertices[i].X.ToString() + " "
                                            + myVertices[i].Y.ToString());
                    for (int i = 0; i < VertexCount; i++)
                    {
                        string line = "";
                        for (int j = 0; j < VertexCount; j++)
                        {
                            line += adjMatrix[i, j] + " ";
                        }
                        sw.WriteLine(line);
                    }
                }
            }
            catch (Exception e)
            {
                MessageBox.Show("Can't save file: " + newFileName + "\r\n" + e.Message, Application.ProductName);
                return false;
            }

            // Clear the dirty bit, cache the new file name and set the caption
            hasBeenModified = false;
            fileName = newFileName;
            if (this.UpdateCaption != null)
                this.UpdateCaption(this, EventArgs.Empty);
            return true;
        }
        public DialogResult Close()
        {
            if (hasBeenModified)
            {
                DialogResult result = MessageBox.Show(fileName, "Do you wish to save?",
                            MessageBoxButtons.YesNoCancel);
                return result;  // yes, no or cancel
            }
            else
                return DialogResult.OK;
        }

        /********************* SELLS/MCVEY ****************************/
        public void CheckRegisterFileExtension()
        {
            if (!this.registerFileExtension || string.IsNullOrEmpty(this.fileExtension)) return;

            // Register custom extension with the shell
            string progID = this.fileExtension + "file";

            using (RegistryKey key = Registry.ClassesRoot.CreateSubKey("." + this.fileExtension))
            {
                // Map custom  extension to a ProgID
                key.SetValue(null, progID);
            }

            // Set ProgID description
            if (!string.IsNullOrEmpty(this.fileExtensionDesc))
            {
                using (RegistryKey key = Registry.ClassesRoot.CreateSubKey(progID))
                {
                    key.SetValue(null, this.fileExtensionDesc);
                }
            }

            // Register open command with the shell
            string cmdkey = progID + @"\shell\open\command";
            using (RegistryKey key = Registry.ClassesRoot.CreateSubKey(cmdkey))
            {
                // Map ProgID to an Open action for the shell
                key.SetValue(null, string.Format("\"{0}\" \"%L\"", Application.ExecutablePath));
            }

            // Register document icon with the shell - doesn't seem to work
            if (string.IsNullOrEmpty(this.iconResourceIndex)) return;
            string icokey = progID + @"\DefaultIcon";
            using (RegistryKey key = Registry.ClassesRoot.CreateSubKey(icokey))
            {
                // Map ProgID to a document icon for the shell
                key.SetValue(null, Application.ExecutablePath + "," + this.iconResourceIndex);
            }
        }

//Algorithms:
        //Resets any var used in running an algorithm back to what it was before the algorith ran
        public void docClearAlgorithm(object sender, EventArgs e)
        {
            parentVertex = -1;
            for (int i = 0; i < MAXVERTICES; i++)
            {
                VertexStates[i] = (int)VertexState.Unvisited;
                vertexColor[i] = Color.Empty;
                distance[i] = int.MaxValue;
                pred[i] = -1;
            }
        }

        //class of what is returned on each step through the algorithms
        public class StepResult
        {
            public int vertex; //current vertex
            public int parent; //parent of cuurent vertex
            public VertexState state; //state of current vertex
            public VertexState prevState; //previous state of vertex
            public string edgeType; //type of edge between parent and current vertex
            public string prevEdgeType; //previous edge type
            public string output; //string that will be added to the algorithm output textbox
            public List<OtherEdge> otherEdges; //List that contains temporary edges - stored so going forwards and backwards is easier
            public struct OtherEdge
            {
                public int start;
                public int end;
                public string edgetype; 
            }
            public StepResult(int parent, int vertex, VertexState state, VertexState prevState, string edgeType, string prevEdgeType, string output, List<OtherEdge> otherEdges = null)
            {
                this.parent = parent;
                this.vertex = vertex;
                this.state = state;
                this.prevState = prevState;
                this.edgeType = edgeType;
                this.prevEdgeType = prevEdgeType;
                this.output = output;
                this.otherEdges = otherEdges ?? new List<OtherEdge>(); //returns empty list if null
            }
        }

        //******************************************************************************/
        //BFS pseudo code: 
        //(from "Data Abstraction & Problem Solving with C++" by Carrano and Henry)
        //BFS(v)
        //  q = a new empty queue
        //  Add v to queue and mark it
        //  q.enqueue(v)
        //  Mark v as visited
        //  while (!q.isEmpty())
        //  {
        //      q.dequeue(w)
        //      Loop invariant: there is a path from vertex w to every vertex in the queue q
        //      for (each unvisited vertex u adjacent to w)
        //      {
        //          Mark u as visited
        //          q.enqueue(u)
        //      }
        //  }
        //******************************************************************************/
        public IEnumerable<StepResult> BFS(int start)
        {
            Queue Q = new Queue();
            string s;
            VertexStates[start] = (int)VertexState.Visited;
            s = "Starting BFS on vertex " + start;
            yield return new StepResult(parentVertex, start, VertexState.Visited, VertexState.Unvisited, treeEdge, initialEdge, s);
            Q.Enqueue(start); 
            while (Q.Count != 0) //while Q is not empty
            {
                int w = (int)Q.Peek(); //int at beginning of queue is the parent
                parentVertex = w;
                for (int i = 0; i < VertexCount; i++)
                {
                    if (AdjMatrixWeight(w, i) != int.MaxValue) //An Edge exists
                    {
                        s = backEdge + ": " + parentVertex + " -> " + i;
                        if (VertexStates[i] == (int)VertexState.Unvisited) //An edge exists from w to i and i is unvisited
                        {
                            Q.Enqueue(i);
                            VertexStates[i] = (int)VertexState.Visited;
                            s = treeEdge + ": " + parentVertex + " -> " + i;
                            yield return new StepResult(parentVertex, i, VertexState.Visited, VertexState.Unvisited, treeEdge, initialEdge, s);
                        }
                         //Other types of edges - BFS so back edges
                        else if (VertexStates[i] == (int)VertexState.Visited) 
                            yield return new StepResult(parentVertex, i, VertexState.Visited, VertexState.Visited, backEdge, initialEdge, s);
                        else if (VertexStates[i] == (int)VertexState.Done) 
                            yield return new StepResult(parentVertex, i, VertexState.Done, VertexState.Done, backEdge, initialEdge, s);
                    }
                }
                Q.Dequeue();
                VertexStates[w] = (int)VertexState.Done;
                s = w + " Done";
                yield return new StepResult(parentVertex, w, VertexState.Done, VertexState.Visited, treeEdge, initialEdge, s);
            }
        }
      
        //******************************************************************************/
        //DFS pseudo code: 
        //(from "Data Abstraction & Problem Solving with C++" by Carrano and Henry)
        //DFS(v)
        //  Mark v as visited
        //  for (each unvisited vertex u adjacent to v)
        //      DFS(u)
        //******************************************************************************/
        public IEnumerable<StepResult> DFS(int start)
        {
            string s;
            int v = start;
            if(parent.Peek() == -1)
                s = "DFS from " + v + ":";
            else
                s = "DFS(" + parent.Peek() + "):  " + treeEdge + " " + parent.Peek() + "->" + v;
            yield return new StepResult(parent.Peek(), start, VertexState.Visited, VertexState.Unvisited, treeEdge, initialEdge, s);
            parent.Push(v);
            //parentVertex = v;
            VertexStates[v] = (int)VertexState.Visited;
            startTimes[v] = time;
            time++;
            for (int i = 0; i < VertexCount; i++)
            {
                if (AdjMatrixWeight(v, i) != int.MaxValue)
                {
                    if (VertexStates[i] == (int)VertexState.Unvisited)  //An edge exists from w to i and i is unvisited
                    {
                        //recursive part
                        foreach (var w in DFS(i))
                        {
                            yield return w;
                        }
                    }
                    //Other types of edges
                    else if (VertexStates[i] == (int)VertexState.Visited) //back edge
                    {
                        s = "DFS(" + parent.Peek() + "):  " + backEdge + " " + parent.Peek() + "->" + i;
                        yield return new StepResult(parent.Peek(), i, VertexState.Visited, VertexState.Visited, backEdge, initialEdge, s);
                    }
                    else if (VertexStates[i] == (int)VertexState.Done && startTimes[i] > startTimes[v]) //forward edge
                    {
                        s = "DFS(" + parent.Peek() + "):  " + forwardEdge + " " + parent.Peek() + "->" + i;
                        yield return new StepResult(parent.Peek(), i, VertexState.Done, VertexState.Done, forwardEdge, initialEdge, s);
                    }
                    else if (VertexStates[i] == (int)VertexState.Done && startTimes[i] < startTimes[v]) //cross edge
                    {
                        s = "DFS(" + parent.Peek() + "):  " + crossEdge + " " + parent.Peek() + "->" + i;
                        yield return new StepResult(parent.Peek(), i, VertexState.Done, VertexState.Done, crossEdge, initialEdge, s);
                    }
                }
            }
            VertexStates[v] = (int)VertexState.Done;
            s = "DFS(" + v + ") done";
            yield return new StepResult(parent.Peek(), v, VertexState.Done, VertexState.Visited, treeEdge, initialEdge, s);
            parent.Pop();
        }
        
        //******************************************************************************/
        //Prim's Algorithm pseudo code:
        //(from "Data Abstraction & Problem Solving with C++" by Carrano and Henry)
        //primsAlgorithm(v)
        //  Mark vertex v as visited and include it in the minimum spanning tree
        //  while (there are unvisited vertices)
        //  {
        //      Find the least-cost edge (v, u) from a visited vertex v to some unvisited vertex u
        //      Mark u as visited
        //      Add the vertex u and the edge (v, u) to the minimum spanning tree
        //  }
        /******************************************************************************/
        public IEnumerable<StepResult> MST(int start) 
        {
            string s;
            List<StepResult.OtherEdge> otherEdges = new List<StepResult.OtherEdge>();
            s = "MST from " + start + ":";
            yield return new StepResult(parentVertex, start, VertexState.Visited, VertexState.Unvisited, treeEdge, initialEdge, s);
            int v = start;
            int totalCost = 0; //cost of whole tree
            Queue Q = new Queue();
            Q.Enqueue(v);
            VertexStates[v] = (int)VertexState.Visited;
            while (Q.Count != 0) //true while there exists connected unvisited vertices
            {
                int next = -1;
                int cost = int.MaxValue;
                for (int i = 0; i < VertexCount; i++)
                {
                    if (VertexStates[i] == (int)VertexState.Visited) //Looks at edges from every visited vertex
                    {
                        for (int u = 0; u < VertexCount; u++)
                        {
                            if (AdjMatrixWeight(i, u) != int.MaxValue && VertexStates[u] == (int)VertexState.Unvisited) //An edge exists from v to u and u is unvisited
                            {
                                if (AdjMatrixWeight(i, u) < cost) //find lowest-cost edge
                                {
                                    cost = AdjMatrixWeight(i, u);
                                    next = u;
                                    parentVertex = i;
                                }
                                s = "Try Edge: " + i + "->" + u + "  cost: " + AdjMatrixWeight(i, u);
                                //Add temporary edges to List otherEdges - to make going forwards and backwards easier
                                otherEdges.Add(new StepResult.OtherEdge { start = i, end = u, edgetype = tempEdge}); 
                                yield return new StepResult(i, u, VertexState.Unvisited, VertexState.Unvisited, tempEdge, initialEdge, s);    //every possible edge from vertex i
                            }
                        }
                    }

                }
                if (next != -1) //An edge from v to u where u is unvisited was found
                {
                    VertexStates[next] = (int)VertexState.Visited;
                    Q.Enqueue(next);
                    s = "Lowest-Cost Edge: " + parentVertex + "->" + next + "  cost: " + AdjMatrixWeight(parentVertex, next);
                    yield return new StepResult(parentVertex, next, VertexState.Visited, VertexState.Unvisited, treeEdge, tempEdge, s, otherEdges); //edge is added to tree
                    otherEdges = new List<StepResult.OtherEdge>();
                    totalCost += AdjMatrixWeight(parentVertex, next);
                }
                else   //No edge from v to u where u is unvisited exists
                {
                    VertexStates[(int)Q.Peek()] = (int)VertexState.Done;
                    s = (int)Q.Peek() + ": No edges to unvisited vertices";
                    yield return new StepResult(parentVertex, (int)Q.Peek(), VertexState.Done, VertexState.Visited, treeEdge, treeEdge, s);
                    Q.Dequeue();
                }

            }
            MessageBox.Show("Total cost of minimum spanning tree: " + totalCost);

        }

        //*******************************************************/
        //Shortest Path - Bellman-Ford-Moore pseudo code
        // https://www.youtube.com/watch?v=iTW2yFYd1Nc
        //  for v in V: (initialize)
        //      v.d = infinity
        //      v.pred = None
        //  for i from 1 to |V|-1
        //      for (u, v) in E:
        //          relax (u, v)
        //  for (u, v) in E: (look for negative-weight cycle)
        //      if v.d > u.d + w(u, v):
        //          report that a negative-weight cycle exists
        //*******************************************************/
        public IEnumerable<StepResult> BFM(int start) //Shortest Path - Bellman-Ford-Moore
        {
            string s;
            s = "Start Shortest Path: (source = " + start + ")";
            yield return new StepResult(parentVertex, start, VertexState.Visited, VertexState.Unvisited, treeEdge, initialEdge, s);
            VertexStates[start] = (int)VertexState.Visited;
            distance[start] = 0;

            for (int i = 0; i < VertexCount - 1; i++)
            {
                for (int v = 0; v < VertexCount; v++)
                {
                    if (VertexStates[v] == (int)VertexState.Visited)
                    {
                        for (int u = 0; u < VertexCount; u++)
                        {
                            if (AdjMatrixWeight(v, u) != int.MaxValue) //An edge exists from v to u
                            {
                                if (AdjMatrixWeight(v, u) + distance[v] < distance[u]) //relax
                                {
                                    distance[u] = AdjMatrixWeight(v, u) + distance[v];
                                    var previousState = (VertexState)VertexStates[u];
                                    VertexStates[u] = (int)VertexState.Visited;
                                    pred[u] = v; //parent of u is v
                                    s = v + "->" + u + ", distance from source: " + distance[u];
                                    yield return new StepResult(v, u, VertexState.Visited, previousState, tempEdge, initialEdge, s);
                                }

                            }
                        }
                    }
                }
            }
            //Look for negative weight cycle
            for (int v = 0; v < VertexCount; v++)
            {
                if (VertexStates[v] == (int)VertexState.Visited)
                {
                    for (int u = 0; u < VertexCount; u++)
                    {
                        if (AdjMatrixWeight(v, u) != int.MaxValue) //An edge exists from v to u
                        {
                            if (AdjMatrixWeight(v, u) + distance[v] < distance[u])
                            {
                                MessageBox.Show("Negative weight cycle exists!", "ERROR");
                                yield break;
                            }

                        }
                    }
                }
            }
            //final results:
            Queue<int> Q = new Queue<int>();
            Q.Enqueue(start);
            while (Q.Count != 0)
            {
                int v = Q.Peek();
                Q.Dequeue();
                for (int i = 0; i < VertexCount; i++)
                {
                    if (pred[i] == v)
                        Q.Enqueue(i); //enqueue the children of v
                }
                VertexStates[v] = (int)VertexState.Done;
                if (v == start)
                    s = "Final Path:";
                else
                    s = pred[v] + "->" + v + ", distance from source: " + distance[v];
                yield return new StepResult(pred[v], v, VertexState.Done, VertexState.Visited, treeEdge, tempEdge, s);
            }

        }
        
    }

}